﻿Imports System.ComponentModel
Imports System.Text
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel

' This sample demostrates the use of a stored procedure query asynchronously

Namespace SilverlightConsole
  Public Class CodeSample
	Public Sub New()
	End Sub

    Public Sub Start()
      AsyncStoredProcQueryCore(_entityManager)
    End Sub

    Private Async Sub AsyncStoredProcQueryCore(ByVal manager As NorthwindIBEntities)
      ' Execute a stored procedure query asynchronously

      SLConsole.WriteLine("Note: The server may take a few seconds to initialize." & vbLf)

      '' Get an Employee so we can pass his ID as a parameter in the stored procedure query
      Dim employee = Await manager.Employees.OrderBy(Function(e) e.LastName).AsScalarAsync().First()

      SLConsole.WriteLine("The employee retrieved is " & employee.FirstName & " " & employee.LastName & "; ID# " & employee.EmployeeID)
      SLConsole.WriteLine("Retrieving " & employee.FirstName & " " & employee.LastName & "'s orders via Stored Procedure Query.")

      ' We can pass the stored proc arguments as parameters to the query.  
      ' Here we'll query for orders for the employee retrieved in the previous query in the year 1996.
      Dim storedProcQuery As StoredProcQuery = manager.EmployeeOrdersForYearQuery(employee.EmployeeID, 1996)

      Dim storedProcQueryResults = Await storedProcQuery.ExecuteAsync()

      Dim orders = New List(Of Order)
      For Each o In storedProcQueryResults
        ' The StoredProcQuery is an IEntityQuery, not an IEntityQuery<T>, so we need to cast the untyped results.
        orders.Add(CType(o, Order))
      Next

      SLConsole.WriteLine("Retrieved " & orders.Count() & " orders:")

      For Each order In orders
        SLConsole.WriteLine("Order " & order.OrderID)
      Next order

      ' Let's also show navigation properties here.  We can't do an Include with the StoredProcQuery, but we can lazily and 
      ' asynchronously load related data. In Silverlight, asynchronous navigation is used to load related data into cache.  
      ' Note that we'll issue two async queries here (via async navigation). Since the queries will complete independent 
      ' of one another, the message list showing the results will not be in any particular order.

      Dim firstOrder = orders(0)
      SLConsole.WriteLine(vbLf & "Retrieving Customer and Line items for first order (" & firstOrder.OrderID & ")...")

      ' Let's get the Customer for the first Order

      Dim cust As Customer = firstOrder.Customer
      AddHandler cust.EntityAspect.PendingEntityResolved, Sub(o, e) SLConsole.WriteLine("Retrieved customer '" & (CType(e.ResolvedEntity, Customer)).CompanyName & "'")


      ' We'll also retrieve the OrderDetails for the first order. We will listen on the PendingEntityListResolved event for such.
      Dim details As RelatedEntityList(Of OrderDetail) = firstOrder.OrderDetails
      AddHandler details.PendingEntityListResolved, Sub(o, e)
                                                      SLConsole.WriteLine("Retrieved " & e.ResolvedEntities.Count & " line items:")
                                                      For Each orderDetail In firstOrder.OrderDetails
                                                        SLConsole.WriteLine("Line item " & orderDetail.OrderID & "," & orderDetail.ProductID)
                                                      Next orderDetail
                                                    End Sub
    End Sub

    Private _entityManager As New NorthwindIBEntities()
  End Class
End Namespace
